"""Tests for the processors for django-mailer."""
from django.test import TestCase
from django.utils.timezone import now, timedelta

from mailer.models import PRIORITY_DEFERRED, PRIORITY_MEDIUM
from mixer.backend.django import mixer

from ..constants import SERVER_STATUS
from ..processors.django_mailer import deferred_emails, email_queue


class MailerDeferredEmailsProcessorTestCase(TestCase):
    """Test case for the ``deferred_emails`` django-mailer processor."""
    longMessage = True

    def setUp(self):
        self.normal_messages = mixer.cycle(10).blend(
            'mailer.Message', priority=PRIORITY_MEDIUM)

    def test_deferred_emails(self):
        self.assertEqual(
            deferred_emails()['status'],
            SERVER_STATUS['OK'],
            msg='Without deferred emails, the status should be OK.'
        )

        mixer.cycle(1).blend('mailer.Message', priority=PRIORITY_DEFERRED)
        self.assertEqual(
            deferred_emails()['status'],
            SERVER_STATUS['WARNING'],
            msg='With 1 deferred email, the status should be WARNING.'
        )

        mixer.cycle(9).blend('mailer.Message', priority=PRIORITY_DEFERRED)
        self.assertEqual(
            deferred_emails()['status'],
            SERVER_STATUS['DANGER'],
            msg='With 10 deferred emails, the status should be DANGER.'
        )


class MailerEmailQueueProcessorTestCase(TestCase):
    """Test case for the ``email_queue`` django-mailer processor."""
    longMessage = True

    def setUp(self):
        self.deferred_messages = mixer.cycle(5).blend(
            'mailer.Message', priority=PRIORITY_DEFERRED)
        self.recently_added = mixer.cycle(5).blend(
            'mailer.Message', when_added=now(),
            priority=PRIORITY_MEDIUM)

    def test_deferred_emails(self):
        self.assertEqual(
            email_queue()['status'],
            SERVER_STATUS['OK'],
            msg='Without queued emails, the status should be OK.'
        )

        mixer.cycle(1).blend('mailer.Message',
                             when_added=now() - timedelta(minutes=40),
                             priority=PRIORITY_MEDIUM)
        self.assertEqual(
            email_queue()['status'],
            SERVER_STATUS['WARNING'],
            msg='With 1 queued email, the status should be WARNING.'
        )

        mixer.cycle(99).blend('mailer.Message',
                              when_added=now() - timedelta(minutes=40),
                              priority=PRIORITY_MEDIUM)
        self.assertEqual(
            email_queue()['status'],
            SERVER_STATUS['DANGER'],
            msg='With 100 queued emails, the status should be DANGER.'
        )



"""Tests for the views of the ``server_guardian_api`` app."""
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin

from .. import views


class ServerGuardianAPIViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``ServerGuardianAPIView`` view class."""
    view_class = views.ServerGuardianAPIView

    def test_view(self):
        self.is_callable(data={'token': 'test_token123'})
        resp = self.get()
        self.assertEqual(
            resp.status_code,
            403,
            msg='Withouth a token, the request should be forbidden.'
        )
        resp = self.get(data={'token': 'wrong_token'})
        self.assertEqual(
            resp.status_code,
            403,
            msg='With an incorrect token, the request should be forbidden.'
        )



"""URLs to run the tests."""
from compat import patterns, include, url
from django.contrib import admin


admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
)



"""Some processors for debugging and testing."""

from ..constants import SERVER_STATUS


def exception_processor():  # pragma: no cover
    """This processor is only useful to test."""
    raise Exception('Something went wrong.')


def dummy_processor():  # pragma: no cover
    """Use this processor to test, that you've set up everything correct."""
    return {
        'label': 'Dummy',
        'status': SERVER_STATUS['OK'],
        'info': 'This is a dummy response.'
    }



"""Default settings for the ``server_guardian_api`` app."""
from django.conf import settings

PROCESSORS = getattr(
    settings,
    'SERVER_GUARDIAN_PROCESSORS',
    ['server_guardian_api.processors.debug.dummy_processor']
)

SECURITY_TOKEN = settings.SERVER_GUARDIAN_SECURITY_TOKEN



"""URLs for the server_guardian_api app."""
from compat import patterns, url

from . import views


urlpatterns = patterns(
    '',
    url(r'^$',
        views.ServerGuardianAPIView.as_view(),
        name='server_guardian_api'),
)



"""Views for the server_guardian_api app."""
from django.views.generic import View
from django.http import HttpResponseForbidden
from django_libs.views_mixins import JSONResponseMixin
from django_libs.loaders import load_member
from .constants import SERVER_STATUS
from .default_settings import PROCESSORS, SECURITY_TOKEN


class ServerGuardianAPIView(JSONResponseMixin, View):
    """
    This view gathers and returns API metrics, that can be returned to the
    ``django-server-guardian``.

    """

    def get(self, request, *args, **kwargs):
        if request.REQUEST.get('token') != SECURITY_TOKEN:
            return HttpResponseForbidden()
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        ctx = []
        for processor_path in PROCESSORS:
            processor = load_member(processor_path)
            try:
                ctx.append(processor())
            except Exception as ex:
                response = {
                    'label': 'error',
                    'status': SERVER_STATUS['DANGER'],
                    'info': (
                        'The server encountered an error while running this'
                        ' processor: "{0}"'.format(ex)
                    )
                }
                ctx.append(response)
        return ctx



"""django-mailer related processors."""

from django.utils.timezone import now, timedelta

from mailer.models import Message, PRIORITY_DEFERRED

from ..constants import SERVER_STATUS

# TODO find way to make these into settings (maybe dict? only clean namespace?)
# From here up it counts as WARNING
DEFERRED_WARNING_THRESHOLD = 1
# From here up it counts as DANGER
DEFERRED_DANGER_THRESHOLD = 10

QUEUE_WARNING_THRESHOLD = 1
QUEUE_DANGER_THRESHOLD = 100
# time in minutes, when an email is considered "stuck" in the queue and should
# have been sent already
QUEUE_TIMEOUT = 20


def deferred_emails():
    """Checks for deferred email, that otherwise fill up the queue."""
    status = SERVER_STATUS['OK']
    count = Message.objects.deferred().count()

    if DEFERRED_WARNING_THRESHOLD <= count < DEFERRED_DANGER_THRESHOLD:
        status = SERVER_STATUS['WARNING']
    if count >= DEFERRED_DANGER_THRESHOLD:
        status = SERVER_STATUS['DANGER']

    return {
        'label': 'Deferred Email',
        'status': status,
        'info': 'There are currently {0} deferred messages.'.format(count)
    }


def email_queue():
    """Checks for emails, that fill up the queue without getting sent."""
    status = SERVER_STATUS['OK']
    count = Message.objects.exclude(priority=PRIORITY_DEFERRED).filter(
        when_added__lte=now() - timedelta(minutes=QUEUE_TIMEOUT)).count()

    if QUEUE_WARNING_THRESHOLD <= count < QUEUE_DANGER_THRESHOLD:
        status = SERVER_STATUS['WARNING']
    if count >= QUEUE_DANGER_THRESHOLD:
        status = SERVER_STATUS['DANGER']

    return {
        'label': 'Queued Email',
        'status': status,
        'info': 'There are currently {0} messages in the mail queue.'.format(
            count)
    }
